import { pgTable, text, serial, integer, boolean, timestamp, real } from "drizzle-orm/pg-core";
import { createInsertSchema } from "drizzle-zod";
import { z } from "zod";

export const incidents = pgTable("incidents", {
  id: serial("id").primaryKey(),
  incidentNumber: text("incident_number").notNull(),
  date: text("date").notNull(), // Storing as text to match CSV format
  time: text("time").notNull(),
  reference: text("reference"),
  location: text("location").notNull(),
  town: text("town"),
  station: text("station"),
  officerInCharge: text("officer_in_charge"),
  watch: text("watch"),
  pumps: real("pumps").default(0),
  waterCarrier: real("water_carrier").default(0),
  supportPump: real("support_pump").default(0),
  aerialLadder: real("aerial_ladder").default(0),
  rescueSupport: real("rescue_support").default(0),
  incidentCommand: real("incident_command").default(0),
  incidentType: text("incident_type"),
  notes: text("notes"),
  safetyCheck: boolean("safety_check").default(false),
  additionalNotes: text("additional_notes"),
  safetyCheckComplete: text("safety_check_complete"),
  createdAt: timestamp("created_at").defaultNow(),
  updatedAt: timestamp("updated_at").defaultNow(),
});

const resourceSchema = z.number().min(0).default(0);
const baseIncidentSchema = {
  incidentNumber: z.string().min(1, "Incident number is required"),
  date: z.string().min(1, "Date is required"),
  time: z.string().min(1, "Time is required"),
  reference: z.string().optional(),
  location: z.string().min(1, "Location is required"),
  town: z.string().optional(),
  station: z.string().optional(),
  officerInCharge: z.string().optional(),
  watch: z.string().optional(),
  pumps: resourceSchema,
  waterCarrier: resourceSchema,
  supportPump: resourceSchema,
  aerialLadder: resourceSchema,
  rescueSupport: resourceSchema,
  incidentCommand: resourceSchema,
  incidentType: z.string().optional(),
  notes: z.string().optional(),
  safetyCheck: z.boolean().default(false),
  additionalNotes: z.string().optional(),
  safetyCheckComplete: z.string().optional(),
};

export const insertIncidentSchema = z.object(baseIncidentSchema);
export const updateIncidentSchema = z.object({
  id: z.number(),
  ...baseIncidentSchema,
});

export type Incident = typeof incidents.$inferSelect;
export type InsertIncident = z.infer<typeof insertIncidentSchema>;
export type UpdateIncident = z.infer<typeof updateIncidentSchema>;

// User schema stays the same as it was
export const users = pgTable("users", {
  id: serial("id").primaryKey(),
  username: text("username").notNull().unique(),
  password: text("password").notNull(),
});

export const insertUserSchema = createInsertSchema(users).pick({
  username: true,
  password: true,
});

export type InsertUser = z.infer<typeof insertUserSchema>;
export type User = typeof users.$inferSelect;

// Define stats schema for dashboard
export const statsSchema = z.object({
  totalIncidents: z.number(),
  byType: z.record(z.string(), z.number()),
  byTown: z.record(z.string(), z.number()),
  byMonth: z.record(z.string(), z.number()),
  byWatch: z.record(z.string(), z.number()),
  byPeriod: z.object({
    weekly: z.number(),
    monthly: z.number()
  }),
  recentIncidents: z.array(z.lazy(() => incidents.$inferSelect)),
});

export type Stats = z.infer<typeof statsSchema>;
